{% extends "admin:admin/base.html" %}
{% load theming_tags %}
{% block blockbots %}
    {{ block.super }}
    {# Using blockbots ensures theming css comes after any form media and other css #}
    {% render_theming_css %}
{% endblock %}
